@@ -45,6 +45,7 @@ macro_rules! try_or_fail {
 	};
 }
 
+#[cfg(feature = "tracing")]
 #[derive(Debug, Copy, Clone)]
 pub struct Snapshot {
 	pub gas_limit: u64,
@@ -134,7 +135,7 @@ impl<'config> Gasometer<'config> {
 	pub fn record_cost(&mut self, cost: u64) -> Result<(), ExitError> {
 		event!(RecordCost {
 			cost,
-			snapshot: self.snapshot()?,
+			snapshot: self.snapshot(),
 		});
 
 		let all_gas_cost = self.total_used_gas() + cost;
@@ -152,7 +153,7 @@ impl<'config> Gasometer<'config> {
 	pub fn record_refund(&mut self, refund: i64) -> Result<(), ExitError> {
 		event!(RecordRefund {
 			refund,
-			snapshot: self.snapshot()?,
+			snapshot: self.snapshot(),
 		});
 
 		self.inner_mut()?.refunded_gas += refund;
@@ -186,7 +187,7 @@ impl<'config> Gasometer<'config> {
 			gas_cost,
 			memory_gas,
 			gas_refund,
-			snapshot: self.snapshot()?,
+			snapshot: self.snapshot(),
 		});
 
 		let all_gas_cost = memory_gas + used_gas + gas_cost;
@@ -210,7 +211,7 @@ impl<'config> Gasometer<'config> {
 	pub fn record_stipend(&mut self, stipend: u64) -> Result<(), ExitError> {
 		event!(RecordStipend {
 			stipend,
-			snapshot: self.snapshot()?,
+			snapshot: self.snapshot(),
 		});
 
 		self.inner_mut()?.used_gas -= stipend;
@@ -248,7 +249,7 @@ impl<'config> Gasometer<'config> {
 
 		event!(RecordTransaction {
 			cost: gas_cost,
-			snapshot: self.snapshot()?,
+			snapshot: self.snapshot(),
 		});
 
 		if self.gas() < gas_cost {
@@ -260,9 +261,9 @@ impl<'config> Gasometer<'config> {
 		Ok(())
 	}
 
-	pub fn snapshot(&self) -> Result<Snapshot, ExitError> {
-		let inner = self.inner.as_ref().map_err(|e| e.clone())?;
-		Ok(Snapshot {
+	#[cfg(feature = "tracing")]
+	pub fn snapshot(&self) -> Option<Snapshot> {
+		self.inner.as_ref().ok().map(|inner| Snapshot {
 			gas_limit: self.gas_limit,
 			memory_gas: inner.memory_gas,
 			used_gas: inner.used_gas,
@@ -460,6 +461,9 @@ pub fn dynamic_opcode_cost<H: Handler>(
 		Opcode::SELFBALANCE if config.has_self_balance => GasCost::Low,
 		Opcode::SELFBALANCE => GasCost::Invalid,
 
+		Opcode::BASEFEE if config.has_base_fee => GasCost::Base,
+		Opcode::BASEFEE => GasCost::Invalid,
+
 		Opcode::EXTCODESIZE => {
 			let target = stack.peek(0)?.into();
 			storage_target = StorageTarget::Address(target);
@@ -831,7 +835,7 @@ impl<'config> Inner<'config> {
 			} => costs::sstore_refund(original, current, new, self.config),
 			GasCost::Suicide {
 				already_removed, ..
-			} => costs::suicide_refund(already_removed),
+			} if !self.config.decrease_clears_refund => costs::suicide_refund(already_removed),
 			_ => 0,
 		}
 	}
